@@ -1,29 +1,18 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# (c) 2012, Stephen Fromm <sfromm@gmail.com>
-# (c) 2016, Toshio Kuratomi <tkuratomi@ansible.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright: (c) 2012, Stephen Fromm <sfromm@gmail.com>
+# Copyright: (c) 2016, Toshio Kuratomi <tkuratomi@ansible.com>
+# Copyright: (c) 2017, Ansible Project
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['stableinterface'],
                     'supported_by': 'core'}
 
-
 DOCUMENTATION = '''
 ---
 module: assemble
@@ -42,34 +31,27 @@ options:
     description:
       - An already existing directory full of source files.
     required: true
-    default: null
-    aliases: []
   dest:
     description:
       - A file to create using the concatenation of all of the source files.
     required: true
-    default: null
   backup:
     description:
       - Create a backup file (if C(yes)), including the timestamp information so
         you can get the original file back if you somehow clobbered it
         incorrectly.
-    required: false
-    choices: [ "yes", "no" ]
-    default: "no"
+    type: bool
+    default: 'no'
   delimiter:
     description:
       - A delimiter to separate the file contents.
     version_added: "1.4"
-    required: false
-    default: null
   remote_src:
     description:
       - If False, it will search for src at originating/master machine, if True it will
         go to the remote/target machine for the src. Default is True.
-    choices: [ "True", "False" ]
-    required: false
-    default: "True"
+    type: bool
+    default: 'yes'
     version_added: "1.4"
   regexp:
     description:
@@ -77,25 +59,23 @@ options:
         all files are assembled. All "\\" (backslash) must be escaped as
         "\\\\" to comply yaml syntax. Uses Python regular expressions; see
         U(http://docs.python.org/2/library/re.html).
-    required: false
-    default: null
   ignore_hidden:
     description:
       - A boolean that controls if files that start with a '.' will be included or not.
-    required: false
-    default: false
+    type: bool
+    default: 'no'
     version_added: "2.0"
   validate:
     description:
       - The validation command to run before copying into place.  The path to the file to
         validate is passed in via '%s' which must be present as in the sshd example below.
         The command is passed securely so shell features like expansion and pipes won't work.
-    required: false
-    default: null
     version_added: "2.0"
-author: "Stephen Fromm (@sfromm)"
+author:
+- Stephen Fromm (@sfromm)
 extends_documentation_fragment:
     - files
+    - decrypt
 '''
 
 EXAMPLES = '''
@@ -119,18 +99,14 @@ EXAMPLES = '''
 
 import codecs
 import os
-import os.path
 import re
 import tempfile
 
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.pycompat24 import get_exception
 from ansible.module_utils.six import b
+from ansible.module_utils._text import to_native
 
 
-# ===========================================
-# Support method
-
 def assemble_from_fragments(src_path, delimiter=None, compiled_regexp=None, ignore_hidden=False):
     ''' assemble a file from a directory of fragments '''
     tmpfd, temp_path = tempfile.mkstemp()
@@ -177,38 +153,37 @@ def cleanup(path, result=None):
     if os.path.exists(path):
         try:
             os.remove(path)
-        except (IOError, OSError):
-            e = get_exception()
+        except (IOError, OSError) as e:
             # don't error on possible race conditions, but keep warning
             if result is not None:
-                result['warnings'] = ['Unable to remove temp file (%s): %s' % (path, str(e))]
+                result['warnings'] = ['Unable to remove temp file (%s): %s' % (path, to_native(e))]
 
 
 def main():
 
     module = AnsibleModule(
         # not checking because of daisy chain to file module
-        argument_spec = dict(
-            src = dict(required=True, type='path'),
-            delimiter = dict(required=False),
-            dest = dict(required=True, type='path'),
+        argument_spec=dict(
+            src=dict(required=True, type='path'),
+            delimiter=dict(required=False),
+            dest=dict(required=True, type='path'),
             backup=dict(default=False, type='bool'),
             remote_src=dict(default=False, type='bool'),
-            regexp = dict(required=False),
-            ignore_hidden = dict(default=False, type='bool'),
-            validate = dict(required=False, type='str'),
+            regexp=dict(required=False),
+            ignore_hidden=dict(default=False, type='bool'),
+            validate=dict(required=False, type='str'),
         ),
-        add_file_common_args=True
+        add_file_common_args=True,
     )
 
-    changed   = False
-    path_hash   = None
-    dest_hash   = None
-    src       = module.params['src']
-    dest      = module.params['dest']
-    backup    = module.params['backup']
+    changed = False
+    path_hash = None
+    dest_hash = None
+    src = module.params['src']
+    dest = module.params['dest']
+    backup = module.params['backup']
     delimiter = module.params['delimiter']
-    regexp    = module.params['regexp']
+    regexp = module.params['regexp']
     compiled_regexp = None
     ignore_hidden = module.params['ignore_hidden']
     validate = module.params.get('validate', None)
@@ -223,9 +198,8 @@ def main():
     if regexp is not None:
         try:
             compiled_regexp = re.compile(regexp)
-        except re.error:
-            e = get_exception()
-            module.fail_json(msg="Invalid Regexp (%s) in \"%s\"" % (e, regexp))
+        except re.error as e:
+            module.fail_json(msg="Invalid Regexp (%s) in \"%s\"" % (to_native(e), regexp))
 
     if validate and "%s" not in validate:
         module.fail_json(msg="validate must contain %%s: %s" % validate)
